from jiemoutils.aliyun.base import RestApi
from jiemoutils.funcs import pymysql
import time


class _SyncDb(RestApi):
    def __init__(self, domain='rds.aliyuncs.com',port=80):
        RestApi.__init__(self,domain, port)
        self.DBInfo = None
        self.DBInstanceId = None
        self.SourceDBInstanceId = None

    def getapiname(self):
        return 'rds.aliyuncs.com.ImportDatabaseBetweenInstances.2014-08-15'


class _DescDb(RestApi):
    def __init__(self, domain='rds.aliyuncs.com',port=80):
        RestApi.__init__(self,domain, port)
        self.Action = 'DescribeDBInstanceAttribute'
        self.DBInstanceId = None

    def getapiname(self):
        return 'rds.aliyuncs.com.ImportDatabaseBetweenInstances.2014-08-15'


def syncdb(dbid_src, dbid_dst, dbname, wait=True):
    sdb = _SyncDb()
    sdb.DBInstanceId = dbid_dst
    sdb.SourceDBInstanceId = dbid_src
    sdb.DBInfo = '{"DBNames":["%s"]}' % dbname
    resp = sdb.getResponse()
    if 'Code' in resp:
        print('error occure ' + str(resp))
    if wait:
        waitdb_synced(dbid_dst)
    return resp

def waitdb_synced(dbid):
    while True:
        time.sleep(10)
        r = descdb(dbid)
        status = r['Items']['DBInstanceAttribute'][0]['DBInstanceStatus']
        print('dbstatus is: ' + status)
        if status != 'ImportingFromOthers':
            break

def descdb(dbid):
    ddb = _DescDb()
    ddb.DBInstanceId = dbid
    return ddb.getResponse()


def cleardb(**dbconf):
    con = pymysql.connect(**
                          dbconf)
    cursor = con.cursor()
    cursor.execute('show tables')
    rs = cursor.fetchall()
    for r in rs:
        cursor.execute('drop table %s' % r[0])



# -*- coding: utf-8 -*-
'''
Created on 2012-7-3

@author: lijie.ma
'''
import sys

try: import httplib
except ImportError:
    import http.client as httplib
import sys
import urllib
import time
import json
import jiemoutils.aliyun as aliyun
import itertools
import mimetypes
import base64
import hmac
import uuid
from hashlib import sha1

def sign(accessKeySecret, parameters):
    #===========================================================================
    # '''签名方法
    # @param secret: 签名需要的密钥
    # @param parameters: 支持字典和string两种
    # '''
    #===========================================================================
    # 如果parameters 是字典类的话
    sortedParameters = sorted(parameters.items(), key=lambda parameters: parameters[0])

    canonicalizedQueryString = ''
    for (k,v) in sortedParameters:
        canonicalizedQueryString += '&' + percent_encode(k) + '=' + percent_encode(v)

    stringToSign = 'POST&%2F&' + percent_encode(canonicalizedQueryString[1:])
    bs = accessKeySecret + "&"
    if sys.version_info.major == 3:
        bs = bytes(bs, encoding='utf8')
        stringToSign = bytes(stringToSign, encoding='utf8')
    h = hmac.new(bs, stringToSign, sha1)
    signature = base64.encodestring(h.digest()).strip()
    return signature

def percent_encode(encodeStr):
    encodeStr = str(encodeStr)
    res = encodeStr
    if sys.version_info.major == 2:
        import urllib
        res = urllib.quote(encodeStr.decode(sys.stdin.encoding).encode('utf8'), '')
    else:
        import urllib.request
        res = urllib.request.quote(encodeStr)
    res = res.replace('+', '%20')
    res = res.replace('*', '%2A')
    res = res.replace('%7E', '~')
    return res

def mixStr(pstr):
    if(isinstance(pstr, str)):
        return pstr
    elif(isinstance(pstr, unicode)):
        return pstr.encode('utf-8')
    else:
        return str(pstr)
    
class FileItem(object):
    def __init__(self,filename=None,content=None):
        self.filename = filename
        self.content = content

class MultiPartForm(object):
    """Accumulate the data to be used when posting a form."""

    def __init__(self):
        self.form_fields = []
        self.files = []
        self.boundary = "PYTHON_SDK_BOUNDARY"
        return
    
    def get_content_type(self):
        return 'multipart/form-data; boundary=%s' % self.boundary

    def add_field(self, name, value):
        """Add a simple field to the form data."""
        self.form_fields.append((name, str(value)))
        return

    def add_file(self, fieldname, filename, fileHandle, mimetype=None):
        """Add a file to be uploaded."""
        body = fileHandle.read()
        if mimetype is None:
            mimetype = mimetypes.guess_type(filename)[0] or 'application/octet-stream'
        self.files.append((mixStr(fieldname), mixStr(filename), mixStr(mimetype), mixStr(body)))
        return
    
    def __str__(self):
        """Return a string representing the form data, including attached files."""
        # Build a list of lists, each containing "lines" of the
        # request.  Each part is separated by a boundary string.
        # Once the list is built, return a string where each
        # line is separated by '\r\n'.  
        parts = []
        part_boundary = '--' + self.boundary
        
        # Add the form fields
        parts.extend(
            [ part_boundary,
              'Content-Disposition: form-data; name="%s"' % name,
              'Content-Type: text/plain; charset=UTF-8',
              '',
              value,
            ]
            for name, value in self.form_fields
            )
        
        # Add the files to upload
        parts.extend(
            [ part_boundary,
              'Content-Disposition: file; name="%s"; filename="%s"' % \
                 (field_name, filename),
              'Content-Type: %s' % content_type,
              'Content-Transfer-Encoding: binary',
              '',
              body,
            ]
            for field_name, filename, content_type, body in self.files
            )
        
        # Flatten the list and add closing boundary marker,
        # then return CR+LF separated data
        flattened = list(itertools.chain(*parts))
        flattened.append('--' + self.boundary + '--')
        flattened.append('')
        return '\r\n'.join(flattened)

class AliyunException(Exception):
    #===========================================================================
    # 业务异常类
    #===========================================================================
    def __init__(self):
        self.code = None
        self.message = None
        self.host = None
        self.requestId = None
    
    def __str__(self, *args, **kwargs):
        sb = "code=" + mixStr(self.code) +\
            " message=" + mixStr(self.message) +\
            " host=" + mixStr(self.host) +\
            " requestId=" + mixStr(self.requestId)
        return sb
       
class RequestException(Exception):
    #===========================================================================
    # 请求连接异常类
    #===========================================================================
    pass

class RestApi(object):
    #===========================================================================
    # Rest api的基类
    #===========================================================================
    
    def __init__(self, domain, port = 80):
        #=======================================================================
        # 初始化基类
        # Args @param domain: 请求的域名或者ip
        #      @param port: 请求的端口
        #=======================================================================
        self.__domain = domain
        self.__port = port
        self.__httpmethod = "POST"
        if(aliyun.getDefaultAppInfo()):
            self.__access_key_id = aliyun.getDefaultAppInfo().accessKeyId
            self.__access_key_secret = aliyun.getDefaultAppInfo().accessKeySecret
        
    def get_request_header(self):
        return {
                 'Content-type': 'application/x-www-form-urlencoded',
                 "Cache-Control": "no-cache",
                 "Connection": "Keep-Alive",
        }
        
    def set_app_info(self, appinfo):
        #=======================================================================
        # 设置请求的app信息
        # @param appinfo: import aliyun
        #                 appinfo aliyun.appinfo(accessKeyId,accessKeySecret)
        #=======================================================================
        self.__access_key_id = appinfo.accessKeyId
        self.__access_key_secret = appinfo.accessKeySecret
        
    def getapiname(self):
        return ""
    
    def getMultipartParas(self):
        return [];

    def getTranslateParas(self):
        return {};
    
    def _check_requst(self):
        pass
    
    def getResponse(self, authrize=None, timeout=30):
        #=======================================================================
        # 获取response结果
        #=======================================================================
        connection = httplib.HTTPConnection(self.__domain, self.__port, timeout)
        timestamp = time.strftime("%Y-%m-%dT%H:%M:%SZ", time.gmtime())
        apiname_split = self.getapiname().split(".")
        parameters = { \
                'Format'        : 'json', \
                'Version'   : apiname_split[4], \
                'Action'        : apiname_split[3], \
                'AccessKeyId'   : self.__access_key_id, \
                'SignatureVersion'  : '1.0', \
                'SignatureMethod'   : 'HMAC-SHA1', \
                'SignatureNonce'    : str(uuid.uuid1()), \
                'TimeStamp'         : timestamp, \
                'partner_id'        : '1.0',\
        }
        application_parameter = self.getApplicationParameters()
        for key in application_parameter.keys():
            parameters[key] = application_parameter[key]

        signature = sign(self.__access_key_secret,parameters)
        parameters['Signature'] = signature
        url = "/?"
        if sys.version_info.major == 2:
            import urllib
            url += urllib.urlencode(parameters)
        else:
            import urllib.parse
            url += urllib.parse.urlencode(parameters)

        connection.connect()
        
        header = self.get_request_header();
        if(self.getMultipartParas()):
            form = MultiPartForm()
            for key in self.getMultipartParas():
                fileitem = getattr(self,key)
                if(fileitem and isinstance(fileitem,FileItem)):
                    form.add_file(key,fileitem.filename,fileitem.content)
            body = str(form)
            header['Content-type'] = form.get_content_type()
        else:
            body = None   
        connection.request(self.__httpmethod, url, body=body, headers=header)
        response = connection.getresponse();
        result = response.read()
        if sys.version_info.major == 3:
            result = str(result, encoding='utf8')
        jsonobj = json.loads(result)
        return jsonobj
    
    
    def getApplicationParameters(self):
        application_parameter = {}
        for key in self.__dict__:
            value = self.__dict__[key]
            if not key.startswith("__") and not key in self.getMultipartParas() and not key.startswith("_RestApi__") and value is not None :
                if(key.startswith("_")):
                    application_parameter[key[1:]] = value
                else:
                    application_parameter[key] = value
        #查询翻译字典来规避一些关键字属性
        translate_parameter = self.getTranslateParas()
        for key in application_parameter:
            if key in translate_parameter:
                application_parameter[translate_parameter[key]] = application_parameter[key]
                del application_parameter[key]
        return application_parameter



from django.db import connection


def raw_query(sql, *arg):
    cur = connection.cursor()
    cur.execute(sql, arg)
    return cur.fetchall()


def query(sql, *arg):
    cur = connection.cursor()
    cur.execute(sql, arg)
    r = cur.fetchall()
    desc = cur.description
    return [dict(zip([col[0] for col in desc], row)) for row in r]


def query_one_row(sql, *arg):
    return query(sql, *arg)[0]


def query_one_col(sql, *arg):
    return [r[0] for r in raw_query(sql, arg)]

def query_value(sql, *arg):
    return raw_query(sql, *arg)[0]



import os
import pymysql


pymysql.install_as_MySQLdb()


def eval_file(filename):
    try:
        f = open(filename).read()
        return eval(f)
    except IOError:
        return {}


def is_private_ip(ip):
    return ip.startswith('192.168.') or ip.startswith('10.') or ip.startswith('127.')


def get_mysql_conf(key):
    return eval_file('/etc/jiemo/mysql.py').get(key, None)


def get_mysql_conf_db(db):
    conf = get_mysql_conf(db)
    if conf:
        conf['db'] = db
    return conf


def get_mysql_django_conf(key):
    conf = get_mysql_conf(key)
    return conf if not conf else {'HOST': conf['host'], 'USER': conf['user'], 'PASSWORD': conf['passwd']}


def get_local_settings(file):
    return eval_file(os.path.join(os.path.dirname(os.path.abspath(file)), 'local_settings.py'))





from django.core.context_processors import csrf
from mako.lookup import TemplateLookup
from mako import exceptions
from django import http
import inspect


def none_empty(o):
    return o if o else ''


def csrf_token(request):
    return '<input type="hidden" name="csrfmiddlewaretoken" value="%s" />' % csrf(request)["csrf_token"]


def model_from_form(obj, form):
    cd = form.cleaned_data
    for v in vars(obj):
        if v in cd:
            setattr(obj, v, cd[v])


class TemplateRender(object):

    def __init__(self, template_dirs, imports):
        myimport = ['from jiemoutils.django_views import none_empty, csrf_token']
        if imports:
            myimport.extend(imports)
        self.lookup = TemplateLookup(
            directories=template_dirs,
            input_encoding='utf-8',
            output_encoding='utf-8',
            default_filters=['none_empty', 'h', ],
            imports=myimport,
        )

    def render_to_response(self, filename, ctx):
        try:
            tp = self.lookup.get_template(filename)
            cont = tp.render(**ctx)
            return http.HttpResponse(cont)
        except :
            return http.HttpResponse(exceptions.html_error_template().render())


class ModuleHandler(object):

    def __init__(self, module):
        fs = inspect.getmembers(module, inspect.isfunction)
        self.funcs = {}
        for func in fs:
            if func[0].startswith('v_'):
                self.funcs[func[0]] = func[1]

    def handle(self, request, func_name):
        fname = 'v_' + func_name
        if fname not in self.funcs:
            return http.HttpResponseNotFound('page not found')
        return self.funcs[fname](request)



import base64
import hmac
from hashlib import sha1
import time
import urllib.request
from jiemoutils import aliyun


HOST="oss.aliyuncs.com"


def _oss_file_url(method, bucket, filename):
    now = int(time.time())
    tnow = now - (now % 1800) + 3800 #避免每个时刻url都不一样
    tosign = "%s\n\n\n%d\n/%s/%s" % (method, tnow, bucket, filename)
    if method == 'PUT' or method == 'POST':
        tnow = int(time.time() + 1000)
        tosign = "%s\n\napplication/octet-stream\n%d\n/%s/%s" % (method, tnow, bucket, filename)
    h = hmac.new(aliyun.getDefaultAppInfo().accessKeySecret.encode(),
             tosign.encode(),
             sha1)
    sign = urllib.request.quote(base64.encodebytes(h.digest()).strip())
    return 'http://%s.oss-cn-hangzhou.aliyuncs.com/%s?OSSAccessKeyId=%s&Expires=%d&Signature=%s' % (
        bucket, filename, aliyun.getDefaultAppInfo().accessKeyId, tnow, sign
    )

def get_file_url(bucket, filename):
    return _oss_file_url('GET', bucket, filename)


def http_put(bucket, filename, cont):
    url = _oss_file_url('PUT', bucket, filename)
    req = urllib.request.Request(url, cont)
    req.get_method = lambda: 'PUT'
    req.add_header('content-type', 'application/octet-stream')
    try:
        return urllib.request.urlopen(req)
    except urllib.request.HTTPError as e:
        print(e)




from setuptools import setup,find_packages

setup(
        name='jiemoutils',
        version='0.7',
        description='jiemo utils',
        author='it_account@jiemodai.com',
        author_email='it_account@jiemodai.com',
        url='http://jiemo.co',
        packages=find_packages(),
        install_requires=['django', 'mako'],
)



# -*- coding: utf-8 -*-
'''
Created on 2012-6-29

@author: lijie.ma
'''
from jiemoutils import funcs

class appinfo(object):
    def __init__(self,accessKeyId,accessKeySecret):
        self.accessKeyId = accessKeyId
        self.accessKeySecret = accessKeySecret

def getDefaultAppInfo():
    pass


def setDefaultAppInfo(accessKeyId,accessKeySecret):
    default = appinfo(accessKeyId,accessKeySecret)
    global getDefaultAppInfo
    getDefaultAppInfo = lambda: default


aliyun_account = funcs.eval_file('/etc/jiemo/aliyun.py')
setDefaultAppInfo(aliyun_account['access_key'], aliyun_account['access_secret'])

